@@ -10,7 +10,7 @@ import databear.databearDB as databearDB
 from databear import sensorfactory
 from databear.errors import DataLogConfigError, MeasureError
 from databear.databearDB import DataBearDB
-from datetime import timedelta
+from datetime import datetime, timedelta
 import concurrent.futures
 import threading #For IPC
 import selectors #For IPC via UDP
@@ -42,12 +42,26 @@ class DataLogger:
         '''
         #Initialize attributes
         self.sensors = {}
+        self.portlocks = {}
         self.loggersettings = [] #Form (<measurement>,<sensor>)
         self.logschedule = schedule.Scheduler()
 
-        #Load hardware driver
-        drivername = os.environ['DBDRIVER']
-        driver_module = importlib.import_module(drivername)
+        #Load driver env var
+        try:
+            drivername = os.environ['DBDRIVER']
+        except KeyError:
+            print('DBDRIVER not set, searching for driver in CWD')
+            drivername = 'dbdriver'
+
+        #Try loading driver as a package, then path
+        try:
+            driver_module = importlib.import_module(drivername)
+        except ImportError:
+            #Try loading from path
+            spec = importlib.util.spec_from_file_location("dbdriver", drivername)
+            driver_module = importlib.util.module_from_spec(spec)
+            spec.loader.exec_module(driver_module)
+
         self.driver = driver_module.dbdriver()
 
         #Set up database connection
@@ -64,37 +78,26 @@ class DataLogger:
 
         #Set up error logging
         logging.basicConfig(
-                format=DataLogger.errorfmt,
-                filename='databear_error.log')
+            level=20,
+            format=DataLogger.errorfmt,
+            filename='databear_error.log'
+            )
 
     def register_sensors(self):
         '''
-        Register sensor classs and load measurements
-        to the measurements table
-        classnames - a list of classnames (which should match module names)
+        Register all sensor classes in sensors_available with the factory
         '''
         #Get sensors from database
-        classnames = self.db.sensors_available
-
-        #Append sys.path with path in DB sensors
-        sensorpath = os.getenv('DBSENSORS')
-        if(sensorpath): sys.path.append(sensorpath)
+        module_names = self.db.sensors_available
 
         #Import all sensor classes
-        for sensorcls in classnames:
-            #Try to import from databear.sensors folder
-            try:
-                impstr = 'databear.sensors.'+sensorcls
-                sensor_module = importlib.import_module(impstr) 
-            except ModuleNotFoundError as mnf:
-                #Check custom sensors folder
-                sensor_module = importlib.import_module(sensorcls)
-
-            sensor_class = getattr(sensor_module,sensorcls)
+        for module_name in module_names:
+            sensor_module = importlib.import_module(module_name)
+            sensor_class = getattr(sensor_module,'dbsensor')
 
             #Register sensor with factory
             sensorfactory.factory.register_sensor(
-                sensorcls,
+                module_name,
                 sensor_class)
 
     def loadconfig(self):
@@ -118,10 +121,10 @@ class DataLogger:
                 sensorsettings['serial_number'],
                 sensorsettings['address'],
                 sensorsettings['virtualport'],
-                sensorsettings['class_name']
+                sensorsettings['module_name'],
+                sensorsettings['sensor_config_id']
                 )
             self.scheduleMeasurement(
-                sensorid,
                 sensorsettings['name'],
                 sensorsettings['measure_interval']
                 )
@@ -136,7 +139,7 @@ class DataLogger:
                 storagesetting['storage_interval'],
                 storagesetting['process'])
                   
-    def addSensor(self,name,sn,address,virtualport,sensortype):
+    def addSensor(self,name,sn,address,virtualport,sensortype,sensorconfigid):
         '''
         Add a sensor to the logger
         '''
@@ -147,19 +150,21 @@ class DataLogger:
             sn,
             address
             )
+        sensor.configid = sensorconfigid
 
         #"Connect" virtual port to hardware using driver
-        #Ignore if port0 (simulated sensors)
-        if virtualport!='port0':
-            hardware_port = self.driver.connect(
-                virtualport,
-                sensor.hardware_settings
-                )
-        else:
-            hardware_port = ''
+        hardware_port = self.driver.connect(virtualport,sensor.hardware_settings)
 
         #"Connect" sensor to hardware
-        sensor.connect(hardware_port)
+        #bus type ports are given thread locks
+        if sensor.uses_portlock: 
+            plock = self.portlocks.get(virtualport)
+            if not plock:
+                plock = threading.Lock()
+                self.portlocks[virtualport] = plock 
+            sensor.connect(hardware_port,plock)
+        else:
+            sensor.connect(hardware_port)
 
         #Add sensor to collection
         self.sensors[name] = sensor
@@ -186,22 +191,41 @@ class DataLogger:
 
         return successflag
     
-    def scheduleMeasurement(self,sensorid,sensorname,interval):
+    def reload(self):
+        successflag = True
+
+        # First stop all current jobs
+        for job in self.logschedule.jobs:
+            self.logschedule.cancel_job(job)
+
+        # Then stop workerpool threads
+        self.workerpool.shutdown()
+
+        # reload configuration, creating sensors as needed
+        self.loadconfig()
+
+        # lastly recreate workerpool based on new number of sensors
+        self.workerpool = concurrent.futures.ThreadPoolExecutor(
+            # use at least 1 worker
+            max_workers=max(len(self.sensors),1))
+
+        return successflag
+
+    def scheduleMeasurement(self,sensorname,interval):
         '''
         Schedule a measurement:
         Interval is seconds
         '''
-        self.sensors[sensorname].configid = sensorid
-
+        
         #Check interval to ensure it isn't too small
         if interval < self.sensors[sensorname].min_interval:
             raise DataLogConfigError('Logger frequency exceeds sensor max')
         
         #Schedule measurement
         m = self.doMeasurement
-        self.logschedule.every(interval).do(m,sensorname)
+        self.logschedule.every(interval).do(m,sensorname,interval)
     
-    def doMeasurement(self,sensorname,storetime,lasttime):
+    def doMeasurement(self,sensorname,interval,scheduled_time,last_time):
         '''
         Perform a measurement on a sensor
         Inputs
@@ -209,9 +233,16 @@ class DataLogger:
         - storetime and lasttime are not currently used here
           but are passed by Schedule when this function is called.
         '''
-        mfuture = self.workerpool.submit(self.sensors[sensorname].measure)
-        mfuture.sname = sensorname
-        mfuture.add_done_callback(self.endMeasurement)
+        #Check to see if job is on time. Skip measurement if
+        #current time - scheduled time is more than the measurement interval
+        dtdiff = datetime.now() - scheduled_time
+        if dtdiff.total_seconds() > interval:
+            #Too late, skip measurement
+            logging.error('Skipping measurement for {}'.format(sensorname))
+        else:
+            mfuture = self.workerpool.submit(self.sensors[sensorname].measure)
+            mfuture.sname = sensorname
+            mfuture.add_done_callback(self.endMeasurement)
         
     def endMeasurement(self,mfuture):
         '''
@@ -226,11 +257,14 @@ class DataLogger:
 
         #Log exceptions
         if merrors:
-            for m in merrors.measurements:
-                logging.error('{}:{} - {}'.format(
-                        merrors.sensor,
-                        m,
-                        merrors.messages[m]))
+            try:
+                for m in merrors.measurements:
+                    logging.error('{}:{} - {}'.format(
+                            merrors.sensor,
+                            m,
+                            merrors.messages[m]))
+            except:
+                raise RuntimeError(merrors)
 
     def scheduleStorage(self,configid,name,sensor,interval,process):
         '''
@@ -242,27 +276,29 @@ class DataLogger:
 
         s = self.storeMeasurement
         #Note: Some parameters for function supplied by Job class in Schedule
-        self.logschedule.every(interval).do(s,configid,name,sensor,process)
+        self.logschedule.every(interval).do(s,configid,name,sensor,process,interval)
 
-    def storeMeasurement(self,logconfigid,name,sensor,process,storetime,lasttime):
+    def storeMeasurement(self,logconfigid,name,sensor,process,interval,scheduled_time,last_time):
         '''
         Store measurement data according to process.
         Inputs
-        - name, sensor
+        - logconfigid for database
+        - name: measurement name
+        - sensor: sensor name
         - process: A valid process type
+        - interval: storage interval (used to determine temporal resolution)
         - storetime: datetime of the scheduled storage
         - lasttime: datetime of last storage event
         - Process = 'average','min','max','dump','sample'
-        - Deletes any data associated with storage after saving
         '''
 
         #Deal with missing last time on start-up
         #Set to storetime - 1 day to ensure all data is included
-        if not lasttime:
-            lasttime = storetime - timedelta(1)
+        if not last_time:
+            last_time = scheduled_time - timedelta(1)
 
         #Get datetimes associated with current storage and prior
-        data = self.sensors[sensor].getdata(name,lasttime,storetime)
+        data = self.sensors[sensor].getdata(name,last_time,scheduled_time)
 
         if not data:
             #No data found to be stored
@@ -271,10 +307,20 @@ class DataLogger:
             return
         
         #Process data
-        storedata = processdata.calculate(process,data,storetime)
+        storedata = processdata.calculate(process,data,scheduled_time)
 
         #Write data to database
-        tresolution = self.sensors[sensor].temporal_resolution
+        if interval < 1:
+            tresolution = 'microseconds'
+        elif interval < 60:
+            tresolution = 'seconds'
+        else:
+            tresolution = 'minutes'
+
+        #Include microseconds if dump is used
+        if process == 'Dump':
+            tresolution = 'microseconds'
+
         for row in storedata:
             dtstr = row[0].isoformat(sep=' ',timespec=tresolution)
             value = row[1]
@@ -314,22 +360,44 @@ class DataLogger:
         msgraw, address = self.udpsocket.recvfrom(1024)
 
         #Decode message
-        msg = json.loads(msgraw)
+        try:
+            msg = json.loads(msgraw)
+        except:
+            logging.warning('Message not valid json, ignoring {}'.format(msgraw))
+            # Create an empty msg dict so we will send invalid command response
+            msg = {}
+            msg['command'] = 'invalid'
+
         if msg['command'] == 'getdata':
             sensorname = msg['arg']
             data = self.sensors[sensorname].getcurrentdata()
             #Convert to JSON appropriate
-            datastr = {}
+            response = {}
             for name, val in data.items():
                 if val:
                     dtstr = val[0].strftime('%Y-%m-%d %H:%M')
-                    datastr[name] = (dtstr,val[1])
+                    response[name] = (dtstr,val[1])
                 else:
-                    datastr[name] = val 
+                    response[name] = val 
                     
-            response = {'response':'OK','data':datastr}
         elif msg['command'] == 'status':
-            response = {'response':'OK'}
+            #Get active sensor names
+            sensornames = list(self.sensors.keys())
+            response = {'status':'running','sensors':sensornames}
+        
+        elif msg['command'] == 'getsensor':
+            '''
+            Return {'measurements':[(measure1,units1),(...)]}
+            '''
+            sensorname = msg['arg']
+            measurelist = []
+            for measurename in self.sensors[sensorname].measurements:
+                measurelist.append(
+                    (measurename,self.sensors[sensorname].units[measurename])
+                    )
+
+            response = {'measurements':measurelist}
+
         elif msg['command'] == 'shutdown':
             self.messages.append(msg['command'])
             response = {'response':'OK'}
@@ -339,6 +407,12 @@ class DataLogger:
                 response = {'response':'OK'}
             else:
                 response = {'response':'Sensor not found'}
+        elif msg['command'] == 'reload':
+            success = self.reload()
+            if success:
+                response = {'response':'OK'}
+            else:
+                response = {'response':'Reload failed'}
         else:
             response = {'response':'Invalid Command'}
             
@@ -363,12 +437,12 @@ class DataLogger:
             # use at least 1 worker
             max_workers=max(len(self.sensors),1))
 
-        while True:
+        logging.info('Starting run loop')
+        exiting = False
+        while not exiting:
             try:
                 self.logschedule.run_pending()
                 sleeptime = self.logschedule.idle_seconds
-                if sleeptime > 0:
-                    time.sleep(sleeptime)
 
                 #Check for messages
                 if self.messages:
@@ -379,10 +453,16 @@ class DataLogger:
                         self.listen=False
                         t.join() #Wait for thread to end
                         print('Shutting down')
+                        # Set exiting to break out of the while loop
+                        exiting = True
+                        # This break only exits the for loop
                         break
-            except AssertionError:
-                logging.error('Measurement too late, logger resetting')
-                self.logschedule.reset()
+
+                #Sleep for maximum of 1 sec
+                if sleeptime > 1: sleeptime = 1
+                if sleeptime < 0: sleeptime = 0
+                time.sleep(sleeptime)
+                
             except KeyboardInterrupt:
                 #Shut down threads
                 self.workerpool.shutdown()
@@ -399,7 +479,7 @@ class DataLogger:
                 raise
 
 
-        #Close CSV after stopping
+        #Close database after stopping
         self.db.close()
       
             
